// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef EasyParser_INCLUDED
#define EasyParser_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

#include "CodingSystem.H"
#include "SgmlParser.H"
#include "UnivCharsetDesc.H"
#include "GrowableVector.H"
#include "ParserOptions.H"
#include "MessageReporter.H"
#include "Boolean.H"
#include "EntityManager.H"

class PosixStorageManager;
class PosixFdStorageManager;
class InputSource;
class streambuf;

class EasyParser : public MessageReporter {
public:
  EasyParser(const char *codingName = 0);
  virtual ~EasyParser();
  void setCodingSystem(const CodingSystem *);
  void setErrorStream(streambuf *);
  // Values for flags
  enum {
    allLinkTypesActivated = 01
    };
  void init(int nFiles, char *const *files,
	    unsigned flags = 0);
  void addCatalogSysid(const char *);
  void addInclude(const char *); // parameter entity to be defined as INCLUDE
  ParserOptions options;
  SgmlParser &parser();
  CString messageText(const char *domain, const char *id);
  const CodingSystem *codingSystem() const;
  EntityManager &entityManager();
  void activateLinkType(const char *);
  InputSource *openDocumentEntity(int nFiles, char *const *files);
private:
  EasyParser(const EasyParser &); // undefined
  void operator=(const EasyParser &); // undefined

  PosixStorageManager *storageManager_;
  PosixFdStorageManager *fdStorageManager_;
  EntityManager entityManager_;
  SgmlParser parser_;
  const CodingSystem *codingSystem_;
  UnivCharsetDesc systemCharset_;
  GrowableVector<const char *> catalogSysids_;
  GrowableVector<const char *> includes_;
  GrowableVector<const char *> activeLinkTypes_;
  IosOutputCharStream errorStream_;
};

inline
SgmlParser &EasyParser::parser()
{
  return parser_;
}

inline
const CodingSystem *EasyParser::codingSystem() const
{
  return codingSystem_;
}

inline
EntityManager &EasyParser::entityManager()
{
  return entityManager_;
}

#endif /* not EasyParser_INCLUDED */
