// Copyright (c) 1995 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif

#include "LineOutputCodingSystem.H"
#include "Owner.H"
#include "CString.H"

class LineEncoder : public Encoder {
public:
  LineEncoder(const OutputCodingSystem *, Char rs, Char re,
	      Char term1, Char term2);
  void output(const Char *, size_t, streambuf *);
private:
  LineEncoder(const LineEncoder &); // undefined
  void operator=(const LineEncoder &); // undefined
  Char rs_, re_;
  Char term1_, term2_;
  Owner<Encoder> sub_;
  CString buf_;
};

LineOutputCodingSystem::LineOutputCodingSystem(const OutputCodingSystem *sub,
					       Char rs, Char re,
					       Char term1, Char term2)
: sub_(sub), rs_(rs), re_(re), term1_(term1), term2_(term2)
{
}

Encoder *LineOutputCodingSystem::makeEncoder() const
{
  return new LineEncoder(sub_, rs_, re_, term1_, term2_);
}

LineEncoder::LineEncoder(const OutputCodingSystem *sub, Char rs, Char re,
			 Char term1, Char term2)
: sub_(sub->makeEncoder()), rs_(rs), re_(re), term1_(term1), term2_(term2)
{
}

void LineEncoder::output(const Char *s, size_t n, streambuf *sb)
{
  buf_.setLength(0);
  for (; n > 0; s++, n--) {
    if (*s == rs_)
      ;
    else if (*s == re_) {
      buf_ += term1_;
      if (term2_)
	buf_ += term2_;
    }
    else
      buf_ += *s;
  }
  sub_->output(buf_.pointer(), buf_.length(), sb);
}
