// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#include "config.h"
#include "MessageEventHandler.H"
#include "SgmlParser.H"
#include "ParserOptions.H"
#ifdef __GNUG__
#include "Entity.H"
#endif

MessageEventHandler::MessageEventHandler(const SgmlParser *parser,
					 MessageReporter *reporter,
					 unsigned flags)
: parser_(parser), reporter_(reporter), errorCount_(0), flags_(flags)
{
}

void MessageEventHandler::subdocEntity(SubdocEntityEvent *event)
{
  const SubdocEntity *entity = event->entity();
  if (entity && (flags_ & parseSubdocs)) {
    ParserSubdocOptions options;
    options.referenced = 1;
    options.inheritIncludes = 1;
    options.inheritActiveLinkTypes = 1;
    ResourcePointer<EntityOrigin> origin(event->entityOrigin()->copy());
    SgmlParser parser(*parser_, *entity, origin, options);
    const SgmlParser *oldParser = parser_;
    parser_ = &parser;
    parser.parseAll(*this);
    parser_ = oldParser;
  }
  delete event;
}

void MessageEventHandler::message(MessageEvent *event)
{
  if (event->message().isError())
    errorCount_++;
  reporter_->reportMessage(event->message());
  delete event;
}
