// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef MessageReporter_INCLUDED
#define MessageReporter_INCLUDED 1

#include "types.H"
#include "MessageBuilder.H"
#include "Boolean.H"
#include "OutputCharStream.H"
#include "Message.H"
#include "Location.H"
#include "CString.H"

class MessageEvent;

class MessageReporter : private MessageBuilder, public Messenger {
public:
  enum Option {
    openElements = 01,
    openEntities = 02
    };
  MessageReporter(OutputCharStream *);
  void reportMessage(const Message &);
  void dispatchMessage(Message &);
  void initMessage(Message &);
  void setCurrentLocation(const Location &);
  void clearCurrentLocation();
  virtual CString messageText(const char *domain, const char *id) = 0;
  void addOption(Option);
private:
  void appendNumber(unsigned long);
  void appendOrdinal(unsigned long);
  void appendChars(const Char *, size_t);
  void appendOther(const OtherMessageArg *);
  void appendText(const char *, const char *);
  const ExternalInfo *locationHeader(const Location &loc, Offset &off);
  void printLocation(const ExternalInfo *info, Offset off);
  OutputCharStream &os();

  OutputCharStream *os_;
  unsigned options_;
  Location currentLocation_;
  static const char domain_[];
};

inline
OutputCharStream &MessageReporter::os()
{
  return *os_;
}

inline
void MessageReporter::setCurrentLocation(const Location &loc)
{
  currentLocation_ = loc;
}

inline
void MessageReporter::clearCurrentLocation()
{
  currentLocation_.clear();
}

#endif /* not MessageReporter_INCLUDED */
