// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef EntityManager_INCLUDED
#define EntityManager_INCLUDED 1

#ifdef __GNUG__
#pragma interface
#endif

#include "Boolean.H"
#include "CString.H"
#include "Vector.H"
#include "types.H"
#include "SubstTable.H"
#include "CopyGrowableVectorD.H"
#include "Location.H"

class ExternalEntityOpener;
class EntityManagerImpl;
class StorageManager;
class InputSource;
class Location;
class InputCodingSystem;
class CharsetInfo;
class ExternalEntity;
class Messenger;
class ExternalInfo;
class UnivCharsetDesc;

struct StorageObjectSpec {
  enum { huge = 01, raw = 02 };
  StorageObjectSpec();
  StorageManager *storageManager;
  const char *codingSystemName;
  const InputCodingSystem *codingSystem;
  CString id;
  unsigned flags;
};

struct ParsedSystemId : public CopyGrowableVectorD<StorageObjectSpec> {
  ParsedSystemId();
  void moveTo(ParsedSystemId &);
  const StorageObjectSpec *defStorageObject() const;
  Location defLocation;
  String<char> canonId() const;
};

struct StorageObjectLocation {
  const StorageObjectSpec *storageObjectSpec;
  unsigned long lineNumber;
  unsigned long columnNumber;
  unsigned long byteIndex;
  unsigned long storageObjectOffset;
};

class EntityManager {
public:
  EntityManager();
  EntityManager(StorageManager *defaultStorageManager,
		const InputCodingSystem *defaultCodingSystem,
		const UnivCharsetDesc &catalogCharset);
  ~EntityManager();
  void init(StorageManager *defaultStorageManager,
	    const InputCodingSystem *defaultCodingSystem,
	    const UnivCharsetDesc &catalogCharset);
  // Returned pointer is valid only while `this' exists.
  ExternalEntityOpener *externalEntityOpener();
  InputSource *openDocumentEntity(const CString *sysids,
				  size_t nSysids,
				  const UnivCharsetDesc &,
				  Messenger &);
  void registerStorageManager(StorageManager *);
  void registerCodingSystem(const char *, const InputCodingSystem *);
  void addCatalogSysid(const CString &str, Boolean mustExist = 0);
  Boolean resolveExternalEntity(const ExternalEntity &entity,
				const CharsetInfo &docCharset,
				const SubstTable<Char> *subst,
				const CString &pero,
				Messenger &,
				ParsedSystemId &);
  InputSource *openParsedSystemId(const ParsedSystemId &,
				  EntityOrigin *,
				  Boolean mayRewind,
				  Messenger &mgr);
  static Boolean externalize(const ExternalInfo *,
			     Offset,
			     StorageObjectLocation &);
  static const ParsedSystemId *
    externalInfoParsedSystemId(const ExternalInfo *);
private:
  EntityManager(const EntityManager &); // undefined
  EntityManager &operator=(const EntityManager &); // undefined
  EntityManagerImpl *impl_;
};

#endif /* not EntityManager_INCLUDED */
