#ifndef PosixBaseStorageObject_INCLUDED
#define PosixBaseStorageObject_INCLUDED 1

#include "StorageManager.H"
#include "Boolean.H"
#include "StringOf.H"
#include <sys/types.h>

class Messenger;

class PosixBaseStorageObject : public StorageObject {
public:
  PosixBaseStorageObject(int fd);
protected:
  size_t getBlockSize() const;
  enum { defaultBlockSize = 8192 };
  int fd_;
  PackedBoolean mayRewind_;
  PackedBoolean eof_;

  void rewindPrepare();
  void saveBytes(const char *, size_t);
  Boolean readSaved(char *, size_t, size_t &);
  Boolean rewind(Messenger &);
  void willNotRewind();
  void unread(const char *s, size_t n);
  virtual Boolean seek(off_t, Messenger &) = 0;
  virtual void close();
  static int xclose(int fd);
private:
  PackedBoolean savingBytes_;
  PackedBoolean readingSaved_;
  off_t startOffset_;
  PackedBoolean canSeek_;
  String<char> savedBytes_;
  size_t nBytesRead_;
};

inline
void PosixBaseStorageObject::saveBytes(const char *s, size_t n)
{
  if (savingBytes_)
    savedBytes_.append(s, n);
}

#endif /* not PosixBaseStorageObject_INCLUDED */
