// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef StorageManager_INCLUDED
#define StorageManager_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

class DescriptorManager;
class Messenger;

#include "CString.H"
#include "types.H"
#include "CharsetInfo.H"
#include <stddef.h>

class StorageManager;
class CharsetInfo;
class Messenger;

class StorageObject {
public:
  StorageObject();
  virtual ~StorageObject();
  virtual Boolean open(Messenger &, Boolean mayRewind,
		       size_t &blockSize) = 0;
  virtual Boolean read(char *buf, size_t bufSize, Messenger &,
		       size_t &nread) = 0;
  virtual Boolean rewind(Messenger &) = 0;
  virtual void willNotRewind();
private:
  StorageObject(const StorageObject &);	// undefined
  void operator=(const StorageObject &); // undefined
};

class StorageManager {
public:
  StorageManager(const UnivCharsetDesc &idCharset);
  virtual StorageObject *makeStorageObject(const CString &id,
					   Messenger &) = 0;
  virtual String<char> canonId(const CString &);
  virtual const char *type() const = 0;
  virtual Boolean inheritable() const;
  virtual ~StorageManager();
  const CharsetInfo &idCharset() const;
  const CString &rsString() const;
  const CString &reString() const;

  /* Try to transform id into an id that will refer to the storage
     object that id would refer to if the ``current directory'' was
     the ``directory'' that the object referred to by specId is in. */

  virtual void mergeId(CString &id, const CString &specId);
protected:
  CString rsString_;
  CString reString_;
private:
  StorageManager(const StorageManager &); // undefined
  void operator=(const StorageManager &); // undefined
  CharsetInfo idCharset_;
};

inline
const CharsetInfo &StorageManager::idCharset() const
{
  return idCharset_;
}

inline
const CString &StorageManager::rsString() const
{
  return rsString_;
}

inline
const CString &StorageManager::reString() const
{
  return reString_;
}

#endif /* not StorageManager_INCLUDED */
