// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef Id_INCLUDED
#define Id_INCLUDED 1

#include "Named.H"
#include "Location.H"
#include "GrowableVectorD.H"

class Id : public Named {
public:
  Id(const CString &);
  void define(const Location &);
  void addPendingRef(const Location &);
  Boolean defined() const;
  const Location &defLocation() const;
  const GrowableVectorD<Location> &pendingRefs() const;
private:
  Location defLocation_;
  GrowableVectorD<Location> pendingRefs_;
};

inline
Id::Id(const CString &name)
: Named(name)
{
}

inline
void Id::define(const Location &loc)
{
  defLocation_ = loc;
  pendingRefs_.clear();
}

inline
Boolean Id::defined() const
{
  return !defLocation_.origin().isNull();
}

inline
const Location &Id::defLocation() const
{
  return defLocation_;
}

inline
const GrowableVectorD<Location> &Id::pendingRefs() const
{
  return pendingRefs_;
}

inline
void Id::addPendingRef(const Location &loc)
{
  pendingRefs_.grow() = loc;
}

#endif /* not Id_INCLUDED */
