// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif
#include "config.h"
#include "Location.H"
#include "Entity.H"

Location::Location()
{
}

Location::Location(Origin *origin, Index i)
: origin_(origin), index_(i)
{
}

Location::Location(ConstResourcePointer<Origin> origin, Index i)
: origin_(origin), index_(i)
{
}

Origin::~Origin()
{
}

const EntityOrigin *Origin::asEntityOrigin() const
{
  return 0;
}

Index Origin::refLength() const
{
  return 0;
}

Boolean Origin::origChar(Char &) const
{
  return 0;
}

Boolean Origin::inBracketedTextOpenDelim() const
{
  return 0;
}

Boolean Origin::inBracketedTextCloseDelim() const
{
  return 0;
}

Boolean Origin::isNumericCharRef(Origin::RefEndType &) const
{
  return 0;
}

BracketOrigin::BracketOrigin(const Location &loc, Position pos)
: loc_(loc), pos_(pos)
{
}

const Location &BracketOrigin::parent() const
{
  return loc_;
}

Boolean BracketOrigin::inBracketedTextOpenDelim() const
{
  return pos_ == open;
}

Boolean BracketOrigin::inBracketedTextCloseDelim() const
{
  return pos_ == close;
}

EntityOrigin::EntityOrigin()
: refEndType_(Origin::endOmitted), refLength_(0)
{
}

EntityOrigin::EntityOrigin(const ConstResourcePointer<Entity> &entity,
			   const Location &refLocation)
: refLocation_(refLocation), refLength_(0), entity_(entity),
  refEndType_(endOmitted), shortrefIndex_(-1)
{
}

EntityOrigin::EntityOrigin(const ConstResourcePointer<Entity> &entity,
			   const Location &refLocation,
			   Index refLength,
			   Origin::RefEndType refEndType,
			   CString &origName,
			   int shortrefIndex)
: refLocation_(refLocation), refLength_(refLength), entity_(entity),
  refEndType_(refEndType), shortrefIndex_(shortrefIndex)
{
  origName.moveTo(origName_);
}

EntityOrigin::~EntityOrigin()
{
}

void EntityOrigin::noteCharRef(Index replacementIndex,
			       const NamedCharRef &ref)
{
  charRefs_.grow();
  charRefs_.last().replacementIndex = replacementIndex;
  charRefs_.last().ref = ref;
  charRefs_.last().origNameOffset = charRefOrigNames_.length();
  charRefOrigNames_ += ref.origName();
}

EntityOrigin::IndexType
EntityOrigin::lookupIndex(Index ind,
			  Offset &charOffset,
			  NamedCharRef &ref) const
{
#if 0
  for (size_t i = 0; i < charRefs_.length(); i++)
    if (ind <= charRefs_[i].replacementIndex)
      break;
#endif
  size_t i;
  // Find i such that
  // charRefs_[I].replacementIndex >= ind
  // charRefs_[i - 1].replacementIndex < ind
  if (charRefs_.length() == 0
      || ind > charRefs_.last().replacementIndex)
    // This will be a common case, so optimize it.
    i = charRefs_.length();
  else {
    // Binary search
    // Invariant:
    // charRefs_ < i have replacementIndex < ind
    // charRefs_ >= lim have replacementIndex >= ind
    i = 0;
    size_t lim = charRefs_.length();
    while (i < lim) {
      size_t mid = i + (lim - i)/2;
      if (charRefs_[mid].replacementIndex >= ind)
	lim = mid;
      else
	i = mid + 1;
    }
  }
  if (i < charRefs_.length() && ind == charRefs_[i].replacementIndex) {
    ref = NamedCharRef(charRefs_[i].ref,
		       charRefOrigNames_.pointer()
		       + charRefs_[i].origNameOffset,
		       (i + 1 < charRefs_.length()
			? charRefs_[i + 1].origNameOffset
			: charRefOrigNames_.length())
		       - charRefs_[i].origNameOffset);
    return replacementCharIndex;
  }
  charOffset = ind - i;
  return normalCharIndex;
}

void EntityOrigin::setExternalInfo(ExternalInfo *info)
{
  externalInfo_ = info;
}

Index EntityOrigin::refLength() const
{
  return refLength_;
}

const Location &EntityOrigin::parent() const
{
  return refLocation_;
}

const EntityOrigin *EntityOrigin::asEntityOrigin() const
{
  return this;
}

NumericCharRefOrigin::NumericCharRefOrigin(const Location &start, Index end,
					   Origin::RefEndType refEndType)
: start_(start), end_(end), refEndType_(refEndType)
{
}

const Location &NumericCharRefOrigin::parent() const
{
  return start_;
}

Index NumericCharRefOrigin::refLength() const
{
  return end_ - start_.index();
}

Boolean
NumericCharRefOrigin::isNumericCharRef(Origin::RefEndType &refEndType) const
{
  refEndType = refEndType_;
  return 1;
}

ReplacementOrigin::ReplacementOrigin(const Location &loc, Char origChar)
: loc_(loc), origChar_(origChar)
{
}

const Location &ReplacementOrigin::parent() const
{
  return loc_;
}

Boolean ReplacementOrigin::origChar(Char &c) const
{
  c = origChar_;
  return 1;
}

ExternalInfo::~ExternalInfo()
{
}

RTTI_DEF0(ExternalInfo)

NamedCharRefBase::NamedCharRefBase()
{
}

NamedCharRefBase::NamedCharRefBase(Index refStartIndex,
				   Origin::RefEndType refEndType)
: refStartIndex_(refStartIndex), refEndType_(refEndType)
{
}

NamedCharRef::NamedCharRef()
{
}

NamedCharRef::NamedCharRef(Index refStartIndex, Origin::RefEndType refEndType,
			   const CString &origName)
: origName_(origName), NamedCharRefBase(refStartIndex, refEndType)
{
}

NamedCharRef:: NamedCharRef(const NamedCharRefBase &base,
			    const Char *s, size_t n)
: NamedCharRefBase(base), origName_(s, n)
{
}

EntityOriginNamedCharRef::EntityOriginNamedCharRef()
{
}

EntityOriginNamedCharRef::EntityOriginNamedCharRef(Index i,
						   const NamedCharRefBase &r,
						   size_t off)
: replacementIndex(i), ref(r), origNameOffset(off)
{
}
