// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif

#include "config.h"
#include "Message.H"
#include "MessageArg.H"

MessageType::MessageType(Severity sev, const char *dom, const char *s,
			 const char *s2)
: severity(sev), domain(dom), id(s), auxId(s2)
{
}

size_t MessageType::nArgs() const
{
  return 0;
}

Boolean MessageType::hasAuxLoc() const
{
  return 0;
}

MessageType0::MessageType0(Severity sev, const char *dom, const char *s)
: MessageType(sev, dom, s)
{
}

MessageType1::MessageType1(Severity sev, const char *dom, const char *s)
: MessageType(sev, dom, s)
{
}

size_t MessageType1::nArgs() const
{
  return 1;
}

MessageType2::MessageType2(Severity sev, const char *dom, const char *s)
: MessageType(sev, dom, s)
{
}

size_t MessageType2::nArgs() const
{
  return 2;
}

MessageType3::MessageType3(Severity sev, const char *dom, const char *s)
: MessageType(sev, dom, s)
{
}

size_t MessageType3::nArgs() const
{
  return 3;
}

MessageType4::MessageType4(Severity sev, const char *dom, const char *s)
: MessageType(sev, dom, s)
{
}

size_t MessageType4::nArgs() const
{
  return 4;
}

MessageType5::MessageType5(Severity sev, const char *dom, const char *s)
: MessageType(sev, dom, s)
{
}

size_t MessageType5::nArgs() const
{
  return 5;
}

MessageType6::MessageType6(Severity sev, const char *dom, const char *s)
: MessageType(sev, dom, s)
{
}

size_t MessageType6::nArgs() const
{
  return 6;
}

MessageType0L::MessageType0L(Severity sev, const char *dom, const char *s,
			     const char *s2)
: MessageType(sev, dom, s, s2)
{
}

Boolean MessageType0L::hasAuxLoc() const
{
  return 1;
}

MessageType1L::MessageType1L(Severity sev, const char *dom, const char *s,
			     const char *s2)
: MessageType(sev, dom, s, s2)
{
}

size_t MessageType1L::nArgs() const
{
  return 1;
}

Boolean MessageType1L::hasAuxLoc() const
{
  return 1;
}



OpenElementInfo::OpenElementInfo()
: included(0), matchIndex(0)
{
}

Message::Message()
{
}

Message::Message(int nArgs)
: args(nArgs)
{
}

void Message::moveTo(Message &to)
{
  to.type = type;
  to.loc = loc;
  to.auxLoc = auxLoc;
  args.moveTo(to.args);
  openElementInfo.moveTo(to.openElementInfo);
}


Messenger::Messenger()
: haveNextLocation_(0)
{
}

Messenger::~Messenger()
{
}

void Messenger::message(const MessageType0 &type)
{
  Message msg(0);
  doInitMessage(msg);
  msg.type = &type;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType1 &type, const MessageArg &arg0)
{
  Message msg(1);
  doInitMessage(msg);
  msg.args[0] = arg0.copy();
  msg.type = &type;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType2 &type,
			const MessageArg &arg0,
			const MessageArg &arg1)
{
  Message msg(2);
  doInitMessage(msg);
  msg.args[0] = arg0.copy();
  msg.args[1] = arg1.copy();
  msg.type = &type;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType3 &type,
			const MessageArg &arg0,
			const MessageArg &arg1,
			const MessageArg &arg2)
{
  Message msg(3);
  doInitMessage(msg);
  msg.args[0] = arg0.copy();
  msg.args[1] = arg1.copy();
  msg.args[2] = arg2.copy();
  msg.type = &type;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType4 &type,
			const MessageArg &arg0,
			const MessageArg &arg1,
			const MessageArg &arg2,
			const MessageArg &arg3)
{
  Message msg(4);
  doInitMessage(msg);
  msg.args[0] = arg0.copy();
  msg.args[1] = arg1.copy();
  msg.args[2] = arg2.copy();
  msg.args[3] = arg3.copy();
  msg.type = &type;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType5 &type,
			const MessageArg &arg0,
			const MessageArg &arg1,
			const MessageArg &arg2,
			const MessageArg &arg3,
			const MessageArg &arg4)
{
  Message msg(5);
  doInitMessage(msg);
  msg.args[0] = arg0.copy();
  msg.args[1] = arg1.copy();
  msg.args[2] = arg2.copy();
  msg.args[3] = arg3.copy();
  msg.args[4] = arg4.copy();
  msg.type = &type;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType6 &type,
			const MessageArg &arg0,
			const MessageArg &arg1,
			const MessageArg &arg2,
			const MessageArg &arg3,
			const MessageArg &arg4,
			const MessageArg &arg5)
{
  Message msg(6);
  doInitMessage(msg);
  msg.args[0] = arg0.copy();
  msg.args[1] = arg1.copy();
  msg.args[2] = arg2.copy();
  msg.args[3] = arg3.copy();
  msg.args[5] = arg5.copy();
  msg.type = &type;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType0L &type, const Location &loc)
{
  Message msg(0);
  doInitMessage(msg);
  msg.type = &type;
  msg.auxLoc = loc;
  dispatchMessage(msg);
}

void Messenger::message(const MessageType1L &type, const MessageArg &arg0,
			const Location &loc)
{
  Message msg(1);
  doInitMessage(msg);
  msg.args[0] = arg0.copy();
  msg.type = &type;
  msg.auxLoc = loc;
  dispatchMessage(msg);
}


void Messenger::setNextLocation(const Location &loc)
{
  haveNextLocation_ = 1;
  nextLocation_ = loc;
}

void Messenger::initMessage(Message &)
{
}

void Messenger::doInitMessage(Message &msg)
{
  initMessage(msg);
  if (haveNextLocation_) {
    msg.loc = nextLocation_;
    haveNextLocation_ = 0;
  }
}

ForwardingMessenger::ForwardingMessenger(Messenger &to)
: to_(&to)
{
}

void ForwardingMessenger::dispatchMessage(Message &msg)
{
  to_->dispatchMessage(msg);
}

void ForwardingMessenger::initMessage(Message &msg)
{
  to_->initMessage(msg);
}

ParentLocationMessenger::ParentLocationMessenger(Messenger &mgr)
: ForwardingMessenger(mgr)
{
}

void ParentLocationMessenger::initMessage(Message &msg)
{
  ForwardingMessenger::initMessage(msg);
  if (!msg.loc.origin().isNull())
    msg.loc = msg.loc.origin()->parent();
}

NullMessenger::NullMessenger()
{
}

void NullMessenger::dispatchMessage(Message &)
{
}
