// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef Message_INCLUDED
#define Message_INCLUDED 1

#ifdef __GNUG__
#pragma interface
#endif

#include <stddef.h>
#include "Location.H"
#include "Vector.H"
#include "Owner.H"
#include "Boolean.H"
#include "CString.H"
#include "MessageArg.H"

struct MessageType {
  enum Severity {
    info,
    warning,
    quantityError,
    error
    };
  MessageType(Severity, const char *, const char *, const char * = 0);
  Severity severity;
  const char *domain;
  const char *id;
  const char *auxId;
  Boolean isError() const;
  virtual size_t nArgs() const;
  virtual Boolean hasAuxLoc() const;
};

struct MessageType0 : public MessageType {
  MessageType0(Severity, const char *, const char *);
};

struct MessageType1 : public MessageType {
  MessageType1(Severity, const char *, const char *);
  size_t nArgs() const;
};

struct MessageType2 : public MessageType {
  MessageType2(Severity, const char *, const char *);
  size_t nArgs() const;
};

struct MessageType3 : public MessageType {
  MessageType3(Severity, const char *, const char *);
  size_t nArgs() const;
};

struct MessageType4 : public MessageType {
  MessageType4(Severity, const char *, const char *);
  size_t nArgs() const;
};

struct MessageType5 : public MessageType {
  MessageType5(Severity, const char *, const char *);
  size_t nArgs() const;
};

struct MessageType6 : public MessageType {
  MessageType6(Severity, const char *, const char *);
  size_t nArgs() const;
};

struct MessageType0L : public MessageType {
  MessageType0L(Severity, const char *, const char *, const char *);
  Boolean hasAuxLoc() const;
};

struct MessageType1L : public MessageType {
  MessageType1L(Severity, const char *, const char *, const char *);
  Boolean hasAuxLoc() const;
  size_t nArgs() const;
};

class OpenElementInfo {
public:
  OpenElementInfo();
  PackedBoolean included;
  CString gi;
  CString matchType;
  unsigned matchIndex;
private:
  OpenElementInfo(const OpenElementInfo &); // undefined
  void operator=(const OpenElementInfo &);  // undefined
};

class Message {
public:
  Message();
  Message(int nArgs);
  const MessageType *type;
  Location loc;
  Location auxLoc;
  Vector<Owner<MessageArg> > args;
  Vector<OpenElementInfo> openElementInfo;
  void moveTo(Message &);
  Boolean isError() const;
private:
  Message(const Message &);	// undefined
  void operator=(const Message &); // undefined
};

class Messenger {
public:
  Messenger();
  virtual ~Messenger();
  void message(const MessageType0 &);
  void message(const MessageType1 &, const MessageArg &);
  void message(const MessageType2 &,
	       const MessageArg &,
	       const MessageArg &);
  void message(const MessageType3 &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &);
  void message(const MessageType4 &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &);
  void message(const MessageType5 &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &);
  void message(const MessageType6 &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &,
	       const MessageArg &);
  void message(const MessageType0L &, const Location &);
  void message(const MessageType1L &, const MessageArg &, const Location &);
  void setNextLocation(const Location &);
  virtual void initMessage(Message &);
  virtual void dispatchMessage(Message &) = 0;
private:
  void doInitMessage(Message &);
  Boolean haveNextLocation_;
  Location nextLocation_;
};

class ForwardingMessenger : public Messenger {
public:
  ForwardingMessenger(Messenger &);
  void dispatchMessage(Message &);
  void initMessage(Message &);
private:
  Messenger *to_;
};

class ParentLocationMessenger : public ForwardingMessenger {
public:
  ParentLocationMessenger(Messenger &);
  void initMessage(Message &);
};

class NullMessenger : public Messenger {
public:
  NullMessenger();
  void dispatchMessage(Message &);
};

inline
Boolean MessageType::isError() const
{
  return severity != info && severity != warning;
}

inline
Boolean Message::isError() const
{
  return type->isError();
}

#endif /* not Message_INCLUDED */
