// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif

#include "config.h"
#include "Parser.H"
#include "ParserMessages.H"

Parser::Parser(ExternalEntityOpener *externalEntityOpener,
	       InputSource *document,
	       const UnivCharsetDesc &initialCharset,
	       const ParserOptions &opt)
: ParserState(externalEntityOpener, opt)
{
  pushInput(document);
  Sd *sd = new Sd(initialCharset);
  sd->setBooleanFeature(Sd::fDATATAG, opt.datatag);
  sd->setBooleanFeature(Sd::fOMITTAG, opt.omittag);
  sd->setBooleanFeature(Sd::fRANK, opt.rank);
  sd->setBooleanFeature(Sd::fSHORTTAG, opt.shorttag);
  sd->setNumberFeature(Sd::fSIMPLE, opt.linkSimple);
  sd->setBooleanFeature(Sd::fIMPLICIT, opt.linkImplicit);
  sd->setNumberFeature(Sd::fEXPLICIT, opt.linkExplicit);
  sd->setNumberFeature(Sd::fCONCUR, opt.concur);
  sd->setNumberFeature(Sd::fSUBDOC, opt.subdoc);
  sd->setBooleanFeature(Sd::fFORMAL, opt.formal);
  setSd(sd);
  setDoFunction(&Parser::doInit);
}

Parser::Parser(const Parser &parent,
	       const SubdocEntity &subdoc,
	       const ResourcePointer<EntityOrigin> &origin,
	       const ParserSubdocOptions &options)
: ParserState(parent, options)
{
  pushInput(externalEntityOpener()
	    ->openExternalEntity(subdoc,
				 origin.pointer(),
				 sd().docCharset(),
				 subdoc.substTable(syntax()),
				 syntax().delimGeneral(Syntax::dPERO),
				 1,
				 messenger()));
  setDoFunction(&Parser::doProlog);
  compilePrologModes();
}

void Parser::giveUp()
{
  if (subdocLevel() > 0)	// FIXME might be subdoc if level == 0
    message(ParserMessages::subdocGiveUp);
  else
    message(ParserMessages::giveUp);
  allDone();
}

Event *Parser::nextEvent()
{
  while (eventQueueEmpty()) {
    if (!haveDoFunction())
      return 0;
    // Use temporary to work round cfront limitation
    DoFunction f = doFunction();
    (this->*f)();
  }
  return eventQueueGet();
}

void Parser::parseAll(EventHandler &handler)
{
  while (!eventQueueEmpty())
    eventQueueGet()->handle(handler);
  // FIXME catch exceptions and reset handler.
  setHandler(&handler);
  while (haveDoFunction()) {
    // Use temporary to work round cfront limitation
    DoFunction f = doFunction();
    (this->*f)();
  }
  unsetHandler();
}
