// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif

#include "config.h"
#include "SgmlParser.H"
#include "Parser.H"

SgmlParser::SgmlParser()
: parser_(0)
{
}

SgmlParser::SgmlParser(ExternalEntityOpener *externalEntityOpener,
		       InputSource *document,
		       const UnivCharsetDesc &docCharset,
		       const ParserOptions &options)
: parser_(new Parser(externalEntityOpener, document, docCharset, options))
{
}

SgmlParser::SgmlParser(const SgmlParser &parent,
		       const SubdocEntity &subdoc,
		       const ResourcePointer<EntityOrigin> &origin,
		       const ParserSubdocOptions &options)
: parser_(new Parser(*parent.parser_, subdoc, origin, options))
{
}

void SgmlParser::init(ExternalEntityOpener *externalEntityOpener,
		      InputSource *document,
		      const UnivCharsetDesc &docCharset,
		      const ParserOptions &options)
{
  delete parser_;
  parser_ = new Parser(externalEntityOpener, document, docCharset, options);
}

void SgmlParser::init(const SgmlParser &parent,
		      const SubdocEntity &subdoc,
		      const ResourcePointer<EntityOrigin> &origin,
		      const ParserSubdocOptions &options)
{
  delete parser_;
  parser_ = new Parser(*parent.parser_, subdoc, origin, options);
}


SgmlParser::~SgmlParser()
{
  delete parser_;
}

Event *SgmlParser::nextEvent()
{
  return parser_->nextEvent();
}

void SgmlParser::parseAll(EventHandler &handler)
{
  parser_->parseAll(handler);
}

ConstResourcePointer<Sd> SgmlParser::sd() const
{
  return parser_->sdPointer();
}

ConstResourcePointer<Syntax> SgmlParser::instanceSyntax() const
{
  return parser_->instanceSyntaxPointer();
}

ConstResourcePointer<Syntax> SgmlParser::prologSyntax() const
{
  return parser_->prologSyntaxPointer();
}

ExternalEntityOpener &SgmlParser::externalEntityOpener() const
{
  return *parser_->externalEntityOpener();
}

void SgmlParser::addInclude(const CString &str)
{
  parser_->addInclude(str);
}

void SgmlParser::activateLinkType(const CString &name)
{
  parser_->activateLinkType(name);
}

void SgmlParser::allLinkTypesActivated()
{
  parser_->allLinkTypesActivated();
}

