// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef SgmlParser_INCLUDED
#define SgmlParser_INCLUDED 1

#ifdef __GNUG__
#pragma interface
#endif

#include "CString.H"

class Event;
class Parser;
class ExternalEntityOpener;
class InputSource;
class UnivCharsetDesc;
class EventHandler;
struct ParserOptions;
struct ParserSubdocOptions;
class SubdocEntity;
template<class T> class ResourcePointer;
template<class T> class ConstResourcePointer;
class EntityOrigin;
class Sd;
class Syntax;

class SgmlParser {
public:
  SgmlParser();			// must call init
  // document will be deleted after use; externalEntityOpener will not.
  SgmlParser(ExternalEntityOpener *externalEntityOpener,
	     InputSource *document,
	     const UnivCharsetDesc &docCharset,
	     const ParserOptions &options);
  void init(ExternalEntityOpener *externalEntityOpener,
	    InputSource *document,
	    const UnivCharsetDesc &docCharset,
	    const ParserOptions &options);
  // for parsing subdocuments
  SgmlParser(const SgmlParser &parent,
	     const SubdocEntity &subdoc,
	     const ResourcePointer<EntityOrigin> &origin,
	     const ParserSubdocOptions &options);
  void init(const SgmlParser &parent,
	    const SubdocEntity &subdoc,
	    const ResourcePointer<EntityOrigin> &origin,
	    const ParserSubdocOptions &);
  ~SgmlParser();
  Event *nextEvent();
  void parseAll(EventHandler &);
  ConstResourcePointer<Sd> sd() const;
  ConstResourcePointer<Syntax> instanceSyntax() const;
  ConstResourcePointer<Syntax> prologSyntax() const;
  ExternalEntityOpener &externalEntityOpener() const;
  void addInclude(const CString &);
  void activateLinkType(const CString &);
  void allLinkTypesActivated();
private:
  SgmlParser(const SgmlParser &);
  void operator=(const SgmlParser &);
  Parser *parser_;
};

#endif /* not SgmlParser_INCLUDED */
