// Copyright (c) 1995 James Clark
// See the file COPYING for copying permission.

#ifndef CopyEventHandler_INCLUDED
#define CopyEventHandler_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

#include "Event.H"
#include "OutputCharStream.H"
#include "GrowableVectorD.H"

// This should be local to CopyEventHandler, but some compilers
// can't handle this.

struct CopyEventHandlerEntityInfo {
  StrOutputCharStream str;
  ConstResourcePointer<EntityOrigin> ref;
};

class CopyEventHandler : public EventHandler {
public:
  enum NormalizeFlags {
    normalizeOmittag = 01,
    normalizeShortref = 02,
    normalizeNet = 04,
    normalizeEmptytag = 010,
    normalizeUnclosed = 020,
    normalizeAttname = 040,
    normalizeAttvalue = 0100,
    normalizeAttspec = 0200,
    normalizeShorttag = (normalizeNet|normalizeEmptytag|normalizeUnclosed
			 |normalizeAttname|normalizeAttvalue|normalizeAttspec),
    normalizeRank = 0400,
    normalizeOmittagHoist = 01000,
    normalizeExpand = 02000,
    normalizeExpandAll = 04000
    };
  CopyEventHandler(OutputCharStream *, unsigned flags,
		   const CString &outputEntity);
  void reOrigin(ReOriginEvent *);
  void sSep(SSepEvent *);
  void ignoredRs(IgnoredRsEvent *);
  void data(DataEvent *);
  void startElement(StartElementEvent *);
  void endElement(EndElementEvent *);
  void sdataEntity(SdataEntityEvent *);
  void externalDataEntity(ExternalDataEntityEvent *);
  void subdocEntity(SubdocEntityEvent *);
  void pi(PiEvent *);
  void endProlog(EndPrologEvent *);
  void commentDecl(CommentDeclEvent *);
  void markedSectionStart(MarkedSectionStartEvent *);
  void markedSectionEnd(MarkedSectionEndEvent *);
  void entityStart(EntityStartEvent *);
  void entityEnd(EntityEndEvent *);
  void ignoredChars(IgnoredCharsEvent *);
  void sgmlDecl(SgmlDeclEvent *);
  void usemap(UsemapEvent *);
  void uselink(UselinkEvent *);
private:
  virtual void reportTagInSpecialMarkedSection(const Location &) = 0;
  OutputCharStream &os();
  void attributeValue(const Text &text);
  void numCharRef(unsigned long n, Origin::RefEndType refEndType,
		  int refLength);
  void withNamedCharRef(const Char *p, size_t n, const Location &loc);
  void writeData(const Char *p, size_t n, const Location &loc);
  void refEnd(Origin::RefEndType refEndType);
  void declaration(const Decl &decl);
  void entityRef(const EntityOrigin *);
  Boolean doNothing(Event *);
  Boolean isSgmlChar(Char);
  const CString &elementTypeOrigName(const ElementType *);
  void handleChange();

  OutputCharStream *os_;
  OutputCharStream *topOs_;
  Boolean inInstance_;
  unsigned entityLevel_;
  unsigned outputEntityLevel_;
  ConstResourcePointer<Syntax> syntax_;
  ConstResourcePointer<Sd> sd_;
  unsigned normalizeFlags_;
  CString outputEntity_;
  GrowableVectorD<CString> elementTypeOrigNames_;
  unsigned omittagHoist_;
  ConstResourcePointer<EntityOrigin> entityOrigin_;
  GrowableVectorD<CopyEventHandlerEntityInfo> entityStack_;
  Boolean inSpecialMarkedSection_;
};

inline
OutputCharStream &CopyEventHandler::os()
{
  return *os_;
}

#endif /* not CopyEventHandler_INCLUDED */
